clear; close all

y_shift = 1;
y_scale = 3;
sample_num = 1e7;

q = 0.5;
p = [q 1-2*q q];

fname_penalty = 'penalty_binary';
fname_penalty_sum = 'penalty_sum';

fname_expected_penalty = 'expected_penalty_discrete';
fname_expected_sum = 'expected_sum_discrete';

dist_matrix = p;
possible_y = round([y_shift y_shift+y_scale y_shift+2*y_scale]);
parameters = [possible_y; dist_matrix];

max_n = 200;
n = 1:max_n;
coefficient = 0.1;
inf_max = 1000;
T = 0;
plot_length = 30;

%Lookup tables to precalculate most stuff.
age_penalty_lut = [-inf_max feval(fname_penalty, n, coefficient)];
expected_sum_lut = expected_sum_generate_lut(age_penalty_lut, max_n);

[~,samples] = histc(rand(1,sample_num+1),cumsum(p(:))/sum(p)); % discrete distributions of 1,6,11
samples = samples'*y_scale+y_shift;
%mean_ = 6;
mean_ = mean(samples);

    
% % Uniform Sampling
% wait_times_uniform = ceil(ones(1,sample_num)*mean(samples));
% S_i = [0 cumsum(wait_times_uniform)];
% D_i = zeros(1,sample_num+1);
% D_i(1) = S_i(1)+samples(1);
% for i = 1:sample_num
%     %Need the extra computation to account for the queueing delay.
%     D_i(i+1) = max(S_i(i+1), D_i(i)) + samples(i+1);
% end
% 
% age_uniform_wait = zeros(1, plot_length);
% i = 1;
% for n = D_i(1):plot_length
%     if n == D_i(i + 1)
%         i = i + 1;
%     end
%     age_uniform_wait(n) = age_penalty_lut(n - S_i(i) + 1);
% end
% 
% color_index = 1;
% plot(0:plot_length-1,-age_uniform_wait(1:plot_length),'b-.','linewidth',2);
% hold on;
% legendInfo{color_index} = 'Uniform Sampling';
% 
% %% Zero Wait
% %Zero wait only works when 1/f_max > E[Y]
% S_i = [0 cumsum(samples(1:sample_num)')];
% D_i = cumsum(samples');
% 
% age_zero_wait = zeros(1, plot_length);
% i = 1;
% for n = D_i(1):plot_length
%     if n == D_i(i + 1)
%         i = i + 1;
%     end
%     age_zero_wait(n) = age_penalty_lut(n - S_i(i) + 1);
% end
% 
% color_index = color_index + 1;
% plot(0:plot_length-1,-age_zero_wait(1:plot_length),'r:o','linewidth',2);
% hold on;
% legendInfo{color_index} = 'Zero Wait';



%% Optimal Policy
optimal_policy_mutual_info;
S_i = [0 cumsum(wait_times_optimal+samples(1:sample_num)')];
D_i = zeros(1,sample_num+1);
D_i(1) = S_i(1)+samples(1);
for i = 1:sample_num
    %Need the extra computation to account for the queueing delay.
    D_i(i+1) = max(S_i(i+1), D_i(i)) + samples(i+1);
end
Y_i = D_i(1:sample_num) - S_i(1:sample_num);

cond_mutual_info = zeros(1, plot_length);
i = 1;
for n = 1:plot_length
    if n == D_i(i + 1)
        i = i + 1;
    end
    cond_mutual_info(n+1) = -feval(fname_expected_penalty, age_penalty_lut, n - S_i(i), 0, parameters);
    if cond_mutual_info(n+1) == inf_max
        n
    end
end

sample_times = NaN(1, plot_length);
delivery_times = NaN(1, plot_length);
n = 1;
while D_i(n) < plot_length
   sample_times(S_i(n) + 1) = -beta;
   delivery_times(D_i(n) + 1) = -beta;
   n = n + 1;
end

line_beta = plot(0:plot_length-1,-beta*ones(1, plot_length),'r--','linewidth',2);
hold on;

i = 0;
for n = 0:plot_length-1
    if ~isnan(delivery_times(n+1))
        string = sprintf('D_{%d}', i - 1);
        line_delivery = line([n n], get(gca, 'ylim'), 'Color', [0 153 76]/255, 'LineStyle', ':', 'linewidth', 2);
        text(n, 0.30, string, 'VerticalAlignment','bottom', 'HorizontalAlignment','center', 'FontSize', 18);
    end
    
    if ~isnan(sample_times(n+1))
        string = sprintf('S_{%d}', i);
        line_sample = line([n n], get(gca, 'ylim'), 'Color', 'b', 'LineStyle', '-.', 'linewidth', 1.5);
        text(n, 0.30, string, 'VerticalAlignment','bottom', 'HorizontalAlignment','center', 'FontSize', 18);
        i = i + 1;
    end
end

line_mutual_info = stem(0:plot_length-1,cond_mutual_info(1:plot_length), 'k.', 'markers', 16,'linewidth',2);
hold on;

% color_index = color_index + 1;
% plot(0:plot_length-1,delivery_times(1:plot_length),'bx','markers',12,'linewidth',2);
% hold on;
% legendInfo{color_index} = 'Delivery Times'; 
% 
% color_index = color_index + 1;
% plot(0:plot_length-1,sample_times(1:plot_length),'c+','markers',12,'linewidth',2);
% hold on;
% legendInfo{color_index} = 'Sample Times'; 


%Mutual Information Graph
axis([0 plot_length/2+5 0 0.30])
xlabel('Time', 'FontSize', 18);
ylabel('Conditional Mutual Information', 'FontSize', 18);
legend([line_mutual_info line_beta line_delivery line_sample], ...
    {'Conditional Mutual Information', 'Beta', 'Delivery Times', 'Sample Times'}, 'FontSize', 18);
saveas(figure(1),'figs/figure5_TimeGraph_binary_discrete.fig');

%Penalty Graph
% figure;
% color_index = 1;
% plot(f_max_values,age_uniform_wait,'b-.','linewidth',2);
% hold on;
% legendInfo{color_index} = 'Uniform Sampling';
% 
% color_index = color_index + 1;
% plot(f_max_values(1:24:end),age_zero_wait(1:24:end),'r:o','linewidth',2);
% hold on;
% legendInfo{color_index} = 'Zero Wait';
% 
% color_index = color_index + 1;
% plot(f_max_values,age_optimal,'k','linewidth',2);
% hold on;
% legendInfo{color_index} = 'Optimal Policy'; 
% 
% %axis([T_min T_max -0.1 0])
% xlabel('f_{max}', 'FontSize', 18);
% ylabel('Average Age Penalty', 'FontSize', 18);
% legend(legendInfo, 'FontSize', 18);
% saveas(figure(2),'MutualInfo_T1_binary_discrete_penalty.fig');